import { PgType, Schema } from 'extract-pg-schema';
import { ConnectionConfig } from 'pg';
import { CompositeProperty } from './generators/composite-types';
import Output from './generators/Output';
import { GenerateIdentifierType, GetMetadata, GetPropertyMetadata } from './metadata';
import TypeMap from './TypeMap';
export declare type InstantiatedConfig = {
    schemas: Record<string, Schema>;
    typeMap: TypeMap;
    getMetadata: GetMetadata;
    getPropertyMetadata: GetPropertyMetadata;
    generateIdentifierType: GenerateIdentifierType;
    propertySortFunction: (a: CompositeProperty, b: CompositeProperty) => number;
    outputPath: string;
    preDeleteOutputFolder: boolean;
    resolveViews: boolean;
};
export declare type PreRenderHook = (outputAcc: Output, instantiatedConfig: InstantiatedConfig) => Output;
export declare type PostRenderHook = (path: string, lines: string[], instantiatedConfig: InstantiatedConfig) => string[];
export declare type Config = {
    connection: string | ConnectionConfig;
    schemas?: string[];
    typeFilter?: (pgType: PgType) => boolean;
    getMetadata?: GetMetadata;
    getPropertyMetadata?: GetPropertyMetadata;
    generateIdentifierType?: GenerateIdentifierType;
    propertySortFunction?: (a: CompositeProperty, b: CompositeProperty) => number;
    outputPath?: string;
    preDeleteOutputFolder?: boolean;
    customTypeMap?: TypeMap;
    resolveViews?: boolean;
    preRenderHooks?: PreRenderHook[];
    postRenderHooks?: PostRenderHook[];
};
