"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultPropertySortFunction = exports.defaultGenerateIdentifierType = exports.defaultGetPropertyMetadata = exports.defaultGetMetadata = void 0;
var recase_1 = require("@kristiandupont/recase");
var path_1 = require("path");
var tagged_comment_parser_1 = require("tagged-comment-parser");
var resolveType_1 = __importDefault(require("./generators/resolveType"));
var toPascalCase = (0, recase_1.recase)(null, 'pascal');
// #region defaultGetMetadata
var defaultGetMetadata = function (details, generateFor, instantiatedConfig) {
    var strippedComment = (0, tagged_comment_parser_1.tryParse)(details.comment).comment;
    var isAgentNoun = ['initializer', 'mutator'].includes(generateFor);
    var relationComment = isAgentNoun
        ? "Represents the ".concat(generateFor, " for the ").concat(details.kind, " ").concat(details.schemaName, ".").concat(details.name)
        : "Represents the ".concat(details.kind, " ").concat(details.schemaName, ".").concat(details.name);
    var suffix = isAgentNoun ? "_".concat(generateFor) : '';
    return {
        name: toPascalCase(details.name + suffix),
        comment: __spreadArray([relationComment], (strippedComment ? [strippedComment] : []), true),
        path: (0, path_1.join)(instantiatedConfig.outputPath, details.schemaName, toPascalCase(details.name)),
    };
};
exports.defaultGetMetadata = defaultGetMetadata;
// #endregion defaultGetMetadata
// #region defaultGetPropertyMetadata
var defaultGetPropertyMetadata = function (property, _details, generateFor, _instantiatedConfig) {
    var strippedComment = (0, tagged_comment_parser_1.tryParse)(property.comment).comment;
    return {
        name: property.name,
        comment: __spreadArray(__spreadArray([], (strippedComment ? [strippedComment] : []), true), (generateFor === 'initializer' && property.defaultValue
            ? ["Default value: ".concat(property.defaultValue)]
            : []), true),
    };
};
exports.defaultGetPropertyMetadata = defaultGetPropertyMetadata;
// #endregion defaultGetPropertyMetadata
// #region defaultGenerateIdentifierType
var defaultGenerateIdentifierType = function (column, details, config) {
    var name = toPascalCase(details.name) + toPascalCase(column.name);
    var innerType = (0, resolveType_1.default)(column, details, __assign(__assign({}, config), { 
        // Explicitly disable identifier resolution so we get the actual inner type here
        generateIdentifierType: undefined }));
    var imports = [];
    var type = innerType;
    if (typeof innerType === 'object') {
        // Handle non-primitives
        type = innerType.name;
        imports.push(innerType);
    }
    return {
        declarationType: 'typeDeclaration',
        name: name,
        exportAs: 'named',
        typeDefinition: ["".concat(type, " & { __brand: '").concat(name, "' }")],
        typeImports: imports,
        comment: ["Identifier type for ".concat(details.schemaName, ".").concat(details.name)],
    };
};
exports.defaultGenerateIdentifierType = defaultGenerateIdentifierType;
// #endregion defaultGenerateIdentifierType
// #region defaultPropertySortFunction
var defaultPropertySortFunction = function (a, b) {
    if (a.isPrimaryKey && !b.isPrimaryKey) {
        return -1;
    }
    else if (!a.isPrimaryKey &&
        b.isPrimaryKey) {
        return 1;
    }
    else {
        // return a.name.localeCompare(b.name);
        return a.ordinalPosition - b.ordinalPosition;
    }
};
exports.defaultPropertySortFunction = defaultPropertySortFunction;
// #endregion defaultPropertySortFunction
