"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var tagged_comment_parser_1 = require("tagged-comment-parser");
var generateProperties_1 = __importDefault(require("./generateProperties"));
var makeMapper = function (config) {
    return function (details) {
        if (details.kind === 'compositeType') {
            // If a composite type has a @type tag in the comment,
            // we will use that type instead of a generated one.
            var tags = (0, tagged_comment_parser_1.tryParse)(details.comment).tags;
            if (tags === null || tags === void 0 ? void 0 : tags.type) {
                return [];
            }
        }
        var declarations = [];
        var _a = config.getMetadata(details, 'selector', config), selectorName = _a.name, selectorComment = _a.comment, path = _a.path;
        if (details.kind === 'table' && config.generateIdentifierType) {
            var columns = details.columns;
            var identifierColumns = columns.filter(function (c) { return c.isPrimaryKey && !c.reference; });
            identifierColumns.forEach(function (c) {
                return declarations.push(config.generateIdentifierType(c, details, config));
            });
        }
        var selectorProperties = (0, generateProperties_1.default)(details, 'selector', config);
        var selectorDeclaration = {
            declarationType: 'interface',
            name: selectorName,
            comment: selectorComment,
            exportAs: 'default',
            properties: selectorProperties,
        };
        declarations.push(selectorDeclaration);
        if (details.kind === 'table') {
            var _b = config.getMetadata(details, 'initializer', config), initializerName = _b.name, initializerComment = _b.comment;
            var initializerProperties = (0, generateProperties_1.default)(details, 'initializer', config);
            var initializerDeclaration = {
                declarationType: 'interface',
                name: initializerName,
                comment: initializerComment,
                exportAs: 'named',
                properties: initializerProperties,
            };
            declarations.push(initializerDeclaration);
            var _c = config.getMetadata(details, 'mutator', config), mutatorName = _c.name, mutatorComment = _c.comment;
            var mutatorProperties = (0, generateProperties_1.default)(details, 'mutator', config);
            var mutatorDeclaration = {
                declarationType: 'interface',
                name: mutatorName,
                comment: mutatorComment,
                exportAs: 'named',
                properties: mutatorProperties,
            };
            declarations.push(mutatorDeclaration);
        }
        return declarations.map(function (declaration) { return ({ path: path, declaration: declaration }); });
    };
};
// "Composite" in this case means tables, views, materialized views and composite types.
// I.e. anything that has "properties" and will be turned into an interface in Typescript.
var makeCompositeGenerator = function (kind, config) {
    return function (schema, outputAcc) {
        var _a, _b;
        var mapper = makeMapper(config);
        var declarations = (_b = (_a = schema["".concat(kind, "s")]) === null || _a === void 0 ? void 0 : _a.map(mapper).flat()) !== null && _b !== void 0 ? _b : [];
        return declarations.reduce(function (acc, _a) {
            var path = _a.path, declaration = _a.declaration;
            var existing = acc[path];
            if (existing) {
                existing.declarations.push(declaration);
            }
            else {
                acc[path] = {
                    declarations: [declaration],
                };
            }
            return acc;
        }, outputAcc);
    };
};
exports.default = makeCompositeGenerator;
