"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tagged_comment_parser_1 = require("tagged-comment-parser");
var makeMapper = function (config) {
    return function (domainDetails) {
        // If a domain has a @type tag in the comment,
        // we will use that type instead of a generated one.
        var tags = (0, tagged_comment_parser_1.tryParse)(domainDetails.comment).tags;
        if (tags === null || tags === void 0 ? void 0 : tags.type) {
            return undefined;
        }
        var _a = config.getMetadata(domainDetails, undefined, config), name = _a.name, comment = _a.comment, path = _a.path;
        var typeDefinition = [];
        var typeImports = [];
        var mapped = config.typeMap[domainDetails.innerType];
        if (!mapped) {
            typeDefinition = ['unknown'];
            console.warn("Domain '".concat(name, "' has unknown type '").concat(domainDetails.innerType, "'"));
        }
        else if (typeof mapped === 'string') {
            typeDefinition = [mapped];
        }
        else {
            typeDefinition = [mapped.name];
            typeImports.push(mapped);
        }
        var declaration = {
            declarationType: 'typeDeclaration',
            name: name,
            comment: comment,
            exportAs: 'default',
            typeDefinition: typeDefinition,
            typeImports: typeImports,
        };
        return { path: path, declaration: declaration };
    };
};
var makeDomainsGenerator = function (config) {
    return function (schema, outputAcc) {
        var _a, _b;
        var declarations = (_b = (_a = schema.domains) === null || _a === void 0 ? void 0 : _a.map(makeMapper(config))) !== null && _b !== void 0 ? _b : [];
        return declarations.reduce(function (acc, elem) {
            if (elem === undefined)
                return acc;
            var path = elem.path, declaration = elem.declaration;
            var existing = acc[path];
            if (existing) {
                existing.declarations.push(declaration);
            }
            else {
                acc[path] = {
                    declarations: [declaration],
                };
            }
            return acc;
        }, outputAcc);
    };
};
exports.default = makeDomainsGenerator;
