"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var tagged_comment_parser_1 = require("tagged-comment-parser");
var escapeName_1 = __importDefault(require("../escapeName"));
var makeMapper = function (style, config) {
    return function (enumDetails) {
        // If an enum has a @type tag in the comment,
        // we will use that type instead of a generated one.
        var tags = (0, tagged_comment_parser_1.tryParse)(enumDetails.comment).tags;
        if (tags === null || tags === void 0 ? void 0 : tags.type) {
            return undefined;
        }
        var _a = config.getMetadata(enumDetails, undefined, config), name = _a.name, comment = _a.comment, path = _a.path;
        if (style === 'type') {
            var declaration = {
                declarationType: 'typeDeclaration',
                name: name,
                comment: comment,
                exportAs: 'default',
                typeDefinition: __spreadArray([
                    ''
                ], enumDetails.values.map(function (value) { return "| '".concat(value, "'"); }), true),
            };
            return { path: path, declaration: declaration };
        }
        else {
            var declaration = {
                declarationType: 'generic',
                comment: comment,
                lines: __spreadArray(__spreadArray([
                    "enum ".concat(name, " {")
                ], enumDetails.values.map(function (value) { return "  ".concat((0, escapeName_1.default)(value), " = '").concat(value, "',"); }), true), [
                    '};',
                    '',
                    "export default ".concat(name, ";"),
                ], false),
            };
            return { path: path, declaration: declaration };
        }
    };
};
var makeEnumsGenerator = function (config) {
    return function (schema, outputAcc) {
        var _a, _b;
        var declarations = (_b = (_a = schema.enums) === null || _a === void 0 ? void 0 : _a.map(makeMapper(config.enumStyle, config))) !== null && _b !== void 0 ? _b : [];
        return declarations.reduce(function (acc, elem) {
            if (elem === undefined)
                return acc;
            var path = elem.path, declaration = elem.declaration;
            var existing = acc[path];
            if (existing) {
                existing.declarations.push(declaration);
            }
            else {
                acc[path] = {
                    declarations: [declaration],
                };
            }
            return acc;
        }, outputAcc);
    };
};
exports.default = makeEnumsGenerator;
