"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tagged_comment_parser_1 = require("tagged-comment-parser");
var resolveTypeFromComment = function (comment) {
    var tags = (0, tagged_comment_parser_1.tryParse)(comment).tags;
    if (tags === null || tags === void 0 ? void 0 : tags.type) {
        if (typeof tags.type === 'string') {
            // If it's just a string, assume system type. No import necessary
            return tags.type;
        }
        else if (Array.isArray(tags.type)) {
            var _a = tags.type, name = _a[0], path = _a[1], isAbsoluteString = _a[2], isDefaultString = _a[3], importAsTypeString = _a[4];
            return {
                name: name,
                path: path,
                isAbsolute: isAbsoluteString === 'true',
                isDefault: isDefaultString === 'true',
                importAsType: importAsTypeString === 'true',
            };
        }
    }
};
var resolveType = function (c, d, config) {
    // 1) Check for a @type tag.
    var typeFromComment = resolveTypeFromComment(c.comment);
    if (typeFromComment) {
        return typeFromComment;
    }
    // 2) If there is a reference, resolve the type from the target
    if (c.reference) {
        var reference_1 = c.reference;
        var target = config.schemas[reference_1.schemaName].tables.find(function (t) { return t.name === reference_1.tableName; });
        if (!target) {
            target = config.schemas[reference_1.schemaName].views.find(function (v) { return v.name === reference_1.tableName; });
        }
        if (!target) {
            target = config.schemas[reference_1.schemaName].materializedViews.find(function (v) { return v.name === reference_1.tableName; });
        }
        if (!target) {
            console.warn('Could not resolve reference', reference_1);
        }
        var column = target.columns.find(function (c) { return c.name === reference_1.columnName; });
        if (column) {
            return resolveType(column, target, config);
        }
    }
    // 3) If this is a view with a source (i.e. the table that it's based on),
    // get the type from the source.
    if (c.source) {
        var source_1 = c.source;
        var target = config.schemas[source_1.schema].tables.find(function (t) { return t.name === source_1.table; });
        if (!target) {
            console.warn('Could not resolve source', source_1);
        }
        var column = target.columns.find(function (c) { return c.name === source_1.column; });
        if (column) {
            return resolveType(column, target, config);
        }
    }
    // 4) if the column is a primary key, use the generated type for it, if we do that
    if (config.generateIdentifierType && c.isPrimaryKey) {
        var path = config.getMetadata(d, 'selector', config).path;
        var _a = config.generateIdentifierType(c, d, config), name = _a.name, exportAs = _a.exportAs;
        return {
            name: name,
            path: path,
            isAbsolute: false,
            isDefault: exportAs === 'default',
            importAsType: true,
        };
    }
    // 5) If there is a typemap type, use that
    if (c.type.fullName in config.typeMap) {
        return config.typeMap[c.type.fullName];
    }
    // 6) If the type is a composite, enum, range or domain, reference that.
    if (['composite', 'enum', 'domain', 'range'].includes(c.type.kind)) {
        var _b = c.type.fullName.split('.'), schemaName = _b[0], typeName_1 = _b[1];
        var target = void 0;
        if (c.type.kind === 'composite') {
            target = config.schemas[schemaName].compositeTypes.find(function (t) { return t.name === typeName_1; });
        }
        else if (c.type.kind === 'enum') {
            target = config.schemas[schemaName].enums.find(function (t) { return t.name === typeName_1; });
        }
        else if (c.type.kind === 'domain') {
            target = config.schemas[schemaName].domains.find(function (t) { return t.name === typeName_1; });
        }
        else if (c.type.kind === 'range') {
            target = config.schemas[schemaName].ranges.find(function (t) { return t.name === typeName_1; });
        }
        if (target) {
            var typeFromComment_1 = resolveTypeFromComment(target.comment);
            if (typeFromComment_1) {
                return typeFromComment_1;
            }
            var _c = config.getMetadata(target, 'selector', config), name = _c.name, path = _c.path;
            return {
                name: name,
                path: path,
                isAbsolute: false,
                isDefault: true,
                importAsType: true,
            };
        }
    }
    // 7) If not found, set to unknown and print a warning.
    console.warn("Could not resolve type ".concat(c.type.fullName, " referenced in ").concat(d.schemaName, ".").concat(c.name));
    return 'unknown';
};
exports.default = resolveType;
