"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var ImportGenerator = /** @class */ (function () {
    /**
     * @param srcPath The path (including filename) of the module we're generating imports for.
     */
    function ImportGenerator(srcPath) {
        this.importMap = {};
        this.srcFolder = path_1.default.dirname(srcPath);
        this.srcModuleName = path_1.default.basename(srcPath);
    }
    ImportGenerator.prototype.addImport = function (_a) {
        var name = _a.name, isDefault = _a.isDefault, absolutePath = _a.path, isAbsolute = _a.isAbsolute, importAsType = _a.importAsType;
        var importPath = absolutePath;
        if (!isAbsolute) {
            var relativePath = path_1.default.relative(this.srcFolder, absolutePath);
            // We never want Windows-style paths in our source. Fix it if necessary.
            if (path_1.default.sep === '\\') {
                relativePath = relativePath.replace(/\\/g, '/');
            }
            if (relativePath[0] !== '.') {
                relativePath = "./".concat(relativePath);
            }
            importPath = relativePath;
        }
        // Ignore imports of "self".
        if (importPath === "./".concat(this.srcModuleName)) {
            return;
        }
        if (!(importPath in this.importMap)) {
            this.importMap[importPath] = { named: new Set(), namedAsType: new Set() };
        }
        var importSet = this.importMap[importPath];
        if (isDefault) {
            if (importAsType) {
                if (importSet.defaultAsType && importSet.defaultAsType !== name) {
                    throw new Error("Multiple default imports attempted: ".concat(importSet.defaultAsType, " and ").concat(name, " from '").concat(importPath, "'"));
                }
                importSet.defaultAsType = name;
            }
            else {
                if (importSet.default && importSet.default !== name) {
                    throw new Error("Multiple default imports attempted: ".concat(importSet.default, " and ").concat(name, " from '").concat(importPath, "'"));
                }
                importSet.default = name;
            }
        }
        else {
            if (importAsType) {
                importSet.namedAsType.add(name);
            }
            else {
                importSet.named.add(name);
            }
        }
    };
    ImportGenerator.prototype.generateLines = function () {
        var _this = this;
        return Object.keys(this.importMap)
            .map(function (relativePath) {
            var _a = _this.importMap[relativePath], defaultImport = _a.default, defaultAsTypeImport = _a.defaultAsType, named = _a.named, namedAsType = _a.namedAsType;
            var lines = [];
            if (defaultImport || named.size > 0) {
                lines.push(_this.generateLine(relativePath, false, defaultImport, Array.from(named)));
            }
            if (defaultAsTypeImport || namedAsType.size > 0) {
                lines.push(_this.generateLine(relativePath, true, defaultAsTypeImport, Array.from(namedAsType)));
            }
            return lines;
        })
            .flat();
    };
    ImportGenerator.prototype.generateLine = function (relativePath, importAsType, defaultImport, namedImports) {
        var importDefaultAsNamed = importAsType && defaultImport && namedImports.length > 0;
        var appliedDefaultImport = importDefaultAsNamed
            ? undefined
            : defaultImport;
        var appliedNamedImports = importDefaultAsNamed
            ? __spreadArray(["default as ".concat(defaultImport)], namedImports, true) : namedImports;
        var allImports = __spreadArray(__spreadArray([], (appliedDefaultImport ? [appliedDefaultImport] : []), true), (appliedNamedImports.length > 0
            ? ["{ ".concat(appliedNamedImports.join(', '), " }")]
            : []), true).join(', ');
        return "import ".concat(importAsType ? 'type ' : '').concat(allImports, " from '").concat(relativePath, "';");
    };
    return ImportGenerator;
}());
exports.default = ImportGenerator;
