"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var escapeName_1 = __importDefault(require("./escapeName"));
var importGenerator_1 = __importDefault(require("./importGenerator"));
var processComments = function (comments, indentation) {
    if (!comments || comments.length === 0) {
        return [];
    }
    var i = ' '.repeat(indentation);
    if (comments.length === 1) {
        return ["".concat(i, "/** ").concat(comments[0], " */")];
    }
    else {
        var lines_1 = [];
        lines_1.push("".concat(i, "/**"));
        comments.forEach(function (comment) { return lines_1.push("".concat(i, " * ").concat(comment)); });
        lines_1.push("".concat(i, " */"));
        return lines_1;
    }
};
var processDeclaration = function (declaration, importGenerator) {
    var declarationLines = [];
    switch (declaration.declarationType) {
        case 'typeDeclaration': {
            var exportAs = declaration.exportAs, name = declaration.name, typeDefinition = declaration.typeDefinition;
            declarationLines.push.apply(declarationLines, processComments(declaration.comment, 0));
            if (exportAs === 'default') {
                if (typeDefinition.length === 1) {
                    declarationLines.push("type ".concat(name, " = ").concat(typeDefinition[0], ";"));
                }
                else {
                    var head = typeDefinition[0], tail_1 = typeDefinition.slice(1);
                    var tailLines = tail_1.map(function (l, i) {
                        return i === tail_1.length - 1 ? "  ".concat(l, ";") : "  ".concat(l);
                    });
                    declarationLines.push.apply(declarationLines, __spreadArray(["type ".concat(name, " = ").concat(head)], tailLines, true));
                }
                declarationLines.push('', "export default ".concat(name, ";"));
            }
            else {
                if (typeDefinition.length === 1) {
                    declarationLines.push("export type ".concat(name, " = ").concat(typeDefinition[0], ";"));
                }
                else {
                    var head = typeDefinition[0], tail_2 = typeDefinition.slice(1);
                    var tailLines = tail_2.map(function (l, i) {
                        return i === tail_2.length - 1 ? "  ".concat(l, ";") : "  ".concat(l);
                    });
                    declarationLines.push.apply(declarationLines, __spreadArray(["export type ".concat(name, " = ").concat(head)], tailLines, true));
                }
            }
            break;
        }
        case 'interface': {
            var exportAs = declaration.exportAs, name = declaration.name, base = declaration.base, properties = declaration.properties, comment = declaration.comment;
            declarationLines.push.apply(declarationLines, (processComments(comment, 0) || []));
            declarationLines.push("export".concat(exportAs === 'default' ? ' default' : '', " interface ").concat(name).concat(base ? " extends ".concat(base) : '', " {"));
            properties.forEach(function (property, index) {
                if (index > 0) {
                    declarationLines.push('');
                }
                declarationLines.push.apply(declarationLines, (processComments(property.comment, 2) || []));
                declarationLines.push("  ".concat((0, escapeName_1.default)(property.name)).concat(property.isOptional ? '?' : '', ": ").concat(property.typeName).concat('[]'.repeat(property.dimensions)).concat(property.isNullable ? ' | null' : '', ";"));
                if (property.typeImports) {
                    property.typeImports.forEach(function (typeImport) {
                        return importGenerator.addImport(typeImport);
                    });
                }
            });
            declarationLines.push('}');
            break;
        }
        case 'generic': {
            declarationLines.push.apply(declarationLines, processComments(declaration.comment, 0));
            declarationLines.push.apply(declarationLines, declaration.lines);
            break;
        }
        default: {
            throw new Error("Unknown declaration type: ".concat(declaration.declarationType));
        }
    }
    if (declaration.typeImports) {
        declaration.typeImports.forEach(function (typeImport) {
            return importGenerator.addImport(typeImport);
        });
    }
    return declarationLines;
};
var render = function (declarations, outputPath) {
    var importGenerator = new importGenerator_1.default(outputPath);
    var lines = [];
    declarations.forEach(function (declaration, index) {
        if (index > 0) {
            lines.push('');
        }
        var declarationLines = processDeclaration(declaration, importGenerator);
        lines.push.apply(lines, declarationLines);
    });
    var importLines = importGenerator.generateLines();
    if (importLines.length) {
        importLines.push('');
    }
    return __spreadArray(__spreadArray([], importLines, true), lines, true);
};
exports.default = render;
