"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var vitest_1 = require("vitest");
var render_1 = __importDefault(require("./render"));
(0, vitest_1.describe)('processGenerationSetup', function () {
    (0, vitest_1.it)('should process a type declaration', function () {
        var declarations = [
            {
                declarationType: 'typeDeclaration',
                name: 'MyString',
                exportAs: 'named',
                comment: ['This is just a string'],
                typeDefinition: ['string'],
            },
        ];
        var lines = (0, render_1.default)(declarations, './');
        (0, vitest_1.expect)(lines).toEqual([
            '/** This is just a string */',
            "export type MyString = string;",
        ]);
    });
    (0, vitest_1.it)('should work with multi-line type definitions', function () {
        var declarations = [
            {
                declarationType: 'typeDeclaration',
                name: 'MyUnion',
                exportAs: 'named',
                comment: [],
                typeDefinition: ['', "| 'apples'", "| 'oranges'", "| 'bananas'"],
            },
        ];
        var lines = (0, render_1.default)(declarations, './');
        (0, vitest_1.expect)(lines).toEqual([
            "export type MyUnion = ",
            "  | 'apples'",
            "  | 'oranges'",
            "  | 'bananas';",
        ]);
    });
    (0, vitest_1.it)('should process an interface', function () {
        var declarations = [
            {
                declarationType: 'interface',
                name: 'Member',
                exportAs: 'default',
                properties: [
                    {
                        name: 'id',
                        typeName: 'MemberId',
                        comment: ['This is the identifier', "it's a number"],
                        dimensions: 0,
                        isOptional: false,
                        isNullable: false,
                    },
                ],
            },
        ];
        var lines = (0, render_1.default)(declarations, './');
        (0, vitest_1.expect)(lines).toEqual([
            "export default interface Member {",
            '  /**',
            '   * This is the identifier',
            "   * it's a number",
            '   */',
            '  id: MemberId;',
            '}',
        ]);
    });
});
